from django import forms
from django.urls import reverse

from wagtail.core import blocks

from .models import BaseForm


class FormChooserBlock(blocks.ChooserBlock):
    target_model = BaseForm
    widget = forms.Select

    def value_for_form(self, value):
        if isinstance(value, self.target_model):
            return value.pk
        return value

    def to_python(self, value):
        # the incoming serialised value should be None or an ID
        if value is None:
            return value
        else:
            try:
                if hasattr(self.target_model.objects, 'get_subclass'):
                    return self.target_model.objects.get_subclass(pk=value)
                return self.target_model.objects.get(pk=value)
            except self.target_model.DoesNotExist:
                return None


class WagtailFormBlock(blocks.StructBlock):
    form = FormChooserBlock()

    class Meta:
        icon = 'icon icon-form'
        template = 'wagtailformblocks/form_block.html'

    def get_action_url(self, form):
        return reverse('wagtailformblocks_process', kwargs={'pk': form.id})

    def get_context(self, value, parent_context=None):
        if not parent_context:
            context = super(WagtailFormBlock, self).get_context(value)
        else:
            context = super(WagtailFormBlock, self).get_context(
                value, parent_context)

        form = value['form']

        context['form'] = form.get_form()
        context['form_id'] = form.id
        context['action_url'] = self.get_action_url(form)
        return context



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-28 12:21
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailformblocks', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='baseform',
            name='add_recaptcha',
            field=models.BooleanField(default=False, help_text='Add a reCapcha field to the form.'),
        ),
    ]



from django.conf import settings
from django.utils.translation import ugettext_lazy as _

DEFAULTS = {
    'SUCCESS_MSG': _('Thank you, the form has been submitted.'),
    'ERROR_MSG': _('There was an error processing the form'),
    'RECAPTCHA_ATTRS': {}
}


def get_formblocks_setting(name):
    return getattr(
        settings, 'WAGTAIL_FORMBLOCKS_'.format(name), DEFAULTS[name])


def recaptcha_enabled():
    return (
        'captcha' in settings.INSTALLED_APPS
        and getattr(settings, 'RECAPTCHA_PUBLIC_KEY', False)
        and getattr(settings, 'RECAPTCHA_PRIVATE_KEY', False)
    )



from django.http import JsonResponse
from django.views.generic import View

from .models import BaseForm
from .utils.conf import get_formblocks_setting


class FormProcessView(View):
    """ Return office information as Json"""
    http_method_names = ['post', ]

    def post(self, request, pk):
        try:
            formdef = BaseForm.objects.get_subclass(id=pk)
        except BaseForm.DoesNotExist:
            err = {
                'message': str(get_formblocks_setting('ERROR_MSG')), # NOQA
                'detail': 'Could not find WagtailForm with id {}'.format(pk)
            }
            return JsonResponse(err, status=400)

        form = formdef.get_form(request.POST)

        if form.is_valid():
            formdef.process_form_submission(form)
        else:
            err = {
                'message': str(get_formblocks_setting('ERROR_MSG')),  # NOQA
                'detail': form.errors
            }
            return JsonResponse(err, status=400)

        resp = {
            'message': formdef.success_message or str(
                get_formblocks_setting('SUCCESS_MSG')
            )
        }
        return JsonResponse(resp)



import json

from django.core.serializers.json import DjangoJSONEncoder
from django.db import models
from django.utils.translation import ugettext_lazy as _
from model_utils.managers import InheritanceManager
from modelcluster.fields import ParentalKey
from modelcluster.models import ClusterableModel
from wagtail.admin.edit_handlers import (
    FieldPanel, FieldRowPanel, InlinePanel, MultiFieldPanel)
from wagtail.admin.utils import send_mail
from wagtail.contrib.forms.models import AbstractFormField

from .forms import FormBuilder
from .utils.conf import recaptcha_enabled


class FormSubmission(models.Model):
    """Data for a Form submission."""

    form_data = models.TextField()
    form = models.ForeignKey('BaseForm', on_delete=models.CASCADE)
    submit_time = models.DateTimeField(
        verbose_name=_('submit time'), auto_now_add=True)

    def get_data(self):
        return json.loads(self.form_data)

    def __str__(self):
        return self.form_data

    class Meta:
        verbose_name = _('form submission')


class FormField(AbstractFormField):
    form = ParentalKey('BaseForm', related_name='form_fields')


class BaseForm(ClusterableModel):
    name = models.CharField(max_length=255)
    store_submission = models.BooleanField(
        default=False,
        help_text=_('Store all form submissions in the database. This has to comply with local privacy laws.') # NOQA
    )
    add_recaptcha = models.BooleanField(
        default=False, help_text=_("Add a reCapcha field to the form."))
    success_message = models.CharField(
        blank=True,
        max_length=255,
        help_text=_('An optional success message to show when the form has been succesfully submitted') # NOQA
    )
    panels = [
        FieldPanel('name',),
        FieldPanel('store_submission',),
        FieldPanel('success_message'),
        InlinePanel('form_fields', label="Form fields"),
    ]

    objects = InheritanceManager()

    def __str__(self):
        return self.name

    def get_form_class(self):
        fb = FormBuilder(
            self.form_fields.all(), add_recaptcha=self.add_recaptcha)
        return fb.get_form_class()

    def get_form_parameters(self):
        return {}

    def get_form(self, *args, **kwargs):
        form_class = self.get_form_class()
        form_params = self.get_form_parameters()
        form_params.update(kwargs)

        return form_class(*args, **form_params)

    def process_form_submission(self, form):
        if self.store_submission:
            return FormSubmission.objects.create(
                form_data=json.dumps(form.cleaned_data, cls=DjangoJSONEncoder),
                form=self)


if recaptcha_enabled():
    BaseForm.panels.insert(2, FieldPanel('add_recaptcha'))


class EmailForm(BaseForm):
    """
    A Form Page that sends email. Pages implementing a form to be send
    to an email should inherit from it
    """

    to_address = models.CharField(
        verbose_name=_('to address'), max_length=255, blank=True,
        help_text=_("Optional - form submissions will be emailed to these addresses. Separate multiple addresses by comma.") # NOQA
    )
    from_address = models.CharField(
        verbose_name=_('from address'), max_length=255, blank=True)
    subject = models.CharField(
        verbose_name=_('subject'), max_length=255, blank=True)

    class Meta:
        verbose_name = _('Email form')

    panels = BaseForm.panels + [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('from_address', classname="col6"),
                FieldPanel('to_address', classname="col6"),
            ]),
            FieldPanel('subject'),
        ], "Email"),
    ]

    def process_form_submission(self, form):
        super(EmailForm, self).process_form_submission(form)

        if self.to_address:
            self.send_form_mail(form)

    def send_form_mail(self, form):
        addresses = [x.strip() for x in self.to_address.split(',')]
        content = []
        for name, field in form.fields.items():
            data = form.cleaned_data.get(name)
            if name == 'recaptcha' or not data:
                continue
            content.append(
                field.label + ': ' + str(data))

        send_mail(
            self.subject, '\n'.join(content), addresses, self.from_address)



from captcha.fields import ReCaptchaField
from wagtail.contrib.forms.forms import FormBuilder as OrigFormBuilder

from wagtailformblocks.utils.conf import (
    get_formblocks_setting, recaptcha_enabled)


class FormBuilder(OrigFormBuilder):

    def __init__(self, fields, **kwargs):
        self.add_recaptcha = kwargs.pop('add_recaptcha')
        super(FormBuilder, self).__init__(fields)

    @property
    def recaptcha_enabled(self):
        return self.add_recaptcha and recaptcha_enabled()

    @property
    def formfields(self):
        formfields = super(FormBuilder, self).formfields

        if self.recaptcha_enabled:
            recaptcha_attrs = get_formblocks_setting('RECAPTCHA_ATTRS')
            formfields['recaptcha'] = ReCaptchaField(**recaptcha_attrs)

        return formfields



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('CHANGELOG.rst') as changelog_file:
    changelog = changelog_file.read()

install_requires = [
    'django-model-utils>=2.5.2',
    'django-recaptcha>=2.0,<2.1',
    'wagtail>=2.0',
]

test_require = [
    # Required for test and coverage
    'pytest',
    'pytest-cov',
    'pytest-django',
    'pytest-pythonpath',
    'coverage',
    'factory-boy',
    'tox',
    # Linting
    'flake8',
    'isort',
]

docs_require = [
    'sphinx',
    'sphinx_rtd_theme',
]

setup(
    name='wagtailformblocks',
    version='0.5.0',
    description="A Wagtail module that provides content blocks to display and process user defined forms", # NOQA
    long_description=readme + '\n\n' + changelog,
    author="Tim Leguijt",
    author_email='info@leguijtict.nl',
    url='https://github.com/LUKKIEN/wagtailformblocks',
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    install_requires=install_requires,
    license='BSD',
    zip_safe=False,
    keywords='wagtailformblocks',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    extras_require={
        'testing': test_require,
        'docs': docs_require,
    },
)



from django.utils.translation import ugettext_lazy as _

from wagtail.contrib.modeladmin.options import (
    ModelAdmin, ModelAdminGroup, modeladmin_register)

from .models import BaseForm, FormSubmission


def _get_valid_subclasses(cls):
    clss = []
    for subcls in cls.__subclasses__():
        if not getattr(subcls, 'wagtailformblocks_autoregister', True) or subcls._meta.abstract:
            pass
        else:
            clss.append(subcls)
        sub_classes = _get_valid_subclasses(subcls)
        if sub_classes:
            clss.extend(sub_classes)
    return clss


all_classes = _get_valid_subclasses(BaseForm)
form_admins = []

for cls in all_classes:
    object_name = cls._meta.object_name
    admin_name = "{}Admin".format(object_name)
    admin_defs = {
        'model': cls,
        'menu_label': cls._meta.verbose_name,
        'menu_icon': 'icon icon-form',
    }
    admin_class = type(admin_name, (ModelAdmin, ), admin_defs)
    form_admins.append(admin_class)


class SubmissionAdmin(ModelAdmin):
    model = FormSubmission
    menu_icon = 'icon icon-table'
    list_display = ('form_data', 'form', 'submit_time')
    list_filter = ('form', )


form_admins.append(SubmissionAdmin)


@modeladmin_register
class FormGroup(ModelAdminGroup):
    """A group where all the contact users go in
       todo: make the group so that we can add language to it
    """
    menu_label = _("User forms")
    menu_icon = 'icon icon-form'
    menu_order = 500
    items = form_admins
